unit LedControl;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MySource;

type
  TStyleColor=(BLANC,BLEUF,BLEUC,VERT,ROUGE,MAUVE,JAUNE);
  TStyleAlign=(AlLeft,AlCenter,AlRight);

type
  TLedControl = class(TGraphicControl)
  private
    { Private declarations }
    FStyleColor:TStyleColor;
    FStyleAlign:TStyleAlign;
    FNumberOfLed:Integer;
    Bitmap1:TBitmap;
    FStyle:array[0..100] of TStyleColor;
    FText,OldText:String;
  protected
    { Protected declarations }
    procedure SetColor(ColorP:TStyleColor);
    procedure SetNumLed(Led:Integer);
    function GetText:String;
    procedure SetText(TextP:String);
    procedure SetAlignText(AlignP:TStyleAlign);
    procedure MyPaint;
    procedure Paint;override;
    procedure GetImage(StyleP:TStyleColor;Pos,LedP:Integer);
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    procedure Clear;
    procedure SetColorLed(LedP:Integer;ColorP:TStyleColor);
  published
    { Published declarations }
    property BitmapColor:TStyleColor read FStyleColor write SetColor;
    property NumberOfLed:Integer read FNumberOfLed write SetNumLed;
    property Text:String read GetText write SetText;
    property AlignText:TStyleAlign read FStyleAlign write SetAlignText;
    property Visible;
  end;


procedure Register;

implementation

{$R LED.RES}

constructor TLedControl.Create(AOwner:TComponent);
var
  i1:Integer;
begin
  inherited Create(AOwner);
  FStyleColor:=VERT;
  FStyleAlign:=AlLeft;
  FNumberOfLed:=5;
  Width:=18*FNumberOfLed;
  Height:=26;
  for i1:=1 to 100 do
  begin
      FStyle[i1]:=VERT;
  end;
  FText:='     ';
  OldText:='';
end;
procedure TLedControl.Clear;
begin
  FText:='';
  OldText:='';
  SetText(FText);
end;
procedure TLedControl.SetColorLed(LedP:Integer;ColorP:TStyleColor);
begin
  FStyle[LedP]:=ColorP;
  Invalidate;
end;
procedure TLedControl.SetColor(ColorP:TStyleColor);
var
  i1:Integer;
begin
  FStyleColor:=ColorP;
  for i1:=1 to 100 do FStyle[i1]:=ColorP;
  Invalidate;
end;
procedure TLedControl.SetNumLed(Led:Integer);
begin
  FNumberOfLed:=Led;
  Width:=FNumberOfLed*18;
  Height:=26;
  OldText:='';
  SetText(FText);
  Invalidate;
end;
function TLedControl.GetText:String;
begin
  Result:=Trim(FText);
end;
procedure TLedControl.SetText(TextP:String);
var
  NombreV:Integer;
begin
  OldText:=FText;
  FText:=TextP;
  if Length(FText)>FNumberOfLed then
  begin
    case FStyleAlign of
       AlLeft:FText:=Copy(FText,1,FNumberOfLed);
       AlRight:FText:=Copy(FText,Length(FText)-FNumberOfLed,FNumberOfLed);
       AlCenter:FText:=Copy(FText,(Length(FText)-FNumberOfLed) div 2,FNumberOfLed);
    end;
  end;
  if Length(FText)<FNumberOfLed then
  begin
    case FStyleAlign of
       AlLeft:FText:=FText+StringOfChar(' ',FNumberOfLed-Length(FText)+1);
       AlRight:FText:=StringOfChar(' ',FNumberOfLed-Length(FText))+FText;
       AlCenter:
       begin
         NombreV:=((FNumberOfLed - Length(FText)) div 2)+Length(FText);
         FText:=StringOfChar(' ',NombreV)+FText;
       end;
    end;
  end;
  MyPaint;
end;
procedure TLedControl.SetAlignText(AlignP:TStyleAlign);
begin
  FStyleAlign:=AlignP;
  Invalidate;
  SetText(FText);
end;
procedure TLedControl.MyPaint;
var
  MeLed,i1:Integer;
  Current1:Char;
begin
  for i1:=1 to FNumberOfLed do
  begin
      MeLed:=-1;
      Current1:=MyUpper(Copy(FText,i1,1))[1];
      if i1>Length(FText) then MeLed:=36;  // if Empty;
      if Current1 in ['A'..'Z'] then MeLed:=Ord(Current1)-Ord('A')+10;
      if Current1 in ['0'..'9'] then MeLed:=Ord(Current1)-Ord('0');
      if MeLed<=-1 then MeLed:=36;
      if Copy(OldText,i1,1)<>Copy(FText,i1,1) then
      begin
          GetImage(FStyle[i1],i1,MeLed);
      end;
  end;
end;
procedure TLedControl.GetImage(StyleP:TStyleColor;Pos,LedP:Integer);
var
  Current1:String;
begin
  case StyleP of
    BLANC:Current1:='BLANC';
    BLEUF:Current1:='BLEUF';
    BLEUC:Current1:='BLEUC';
    VERT:Current1:='VERT';
    ROUGE:Current1:='ROUGE';
    MAUVE:Current1:='MAUVE';
    JAUNE:Current1:='JAUNE';
  end;
  Bitmap1:=TBitmap.Create;
  try
  begin
    Bitmap1.LoadFromResourceName(HInstance,Current1);
    Canvas.CopyRect(Rect((Pos-1)*18,0,(Pos*18),26),Bitmap1.Canvas,Rect((LedP)*18,0,(LedP+1)*18,26));
    Bitmap1.FreeImage;
    Bitmap1.Free;
  end;
  except
    Bitmap1.Free;
  end;
end;
procedure TLedControl.Paint;
begin
  OldText:='';
  MyPaint;
end;


procedure Register;
begin
  RegisterComponents('MyControl1', [TLedControl]);
end;

end.
